@@ -1391,7 +1391,45 @@ class ApiContentFeedback(Resource):
             _doc['id'] = _id       
             return _doc
 
-        return {'ok': True, 'feedback': [process_feedback(item.to_dict(), item._id) for item in users], 'total': total}
+        casinos, _ = Page.get(
+            category='provider',
+            is_redirect=False, 
+            locale=current_app.config['BABEL_DEFAULT_LOCALE'],
+            _process=False,
+            _all=True,
+            _source=[
+                'alias', 
+                'title',
+                'comments',
+                'category',
+                'locale',
+            ]
+        )
+
+        def process_approved(_doc):
+            if 'comment_pros' in _doc:
+                _doc['comment_pros'] = '<br />'.join(_doc['comment_pros'].split('\n'))   
+            if 'comment_cons' in _doc:
+                _doc['comment_cons'] = '<br />'.join(_doc['comment_cons'].split('\n'))   
+            _doc['country'] = _doc['country'].upper()
+            return _doc
+
+        _approved = []
+        for item in casinos:
+            if len(item['_source']['comments']):
+                _d2 = {
+                    'casino': item['_source']['title'], 
+                    'full_path': f"https://{current_app.config['DOMAIN']}{get_page_url(item['_source'])}"
+                }
+                _approved += [process_approved(dict(_d1, **_d2)) for _d1 in item['_source']['comments']]
+
+        return {
+            'ok': True, 
+            'feedback': [process_feedback(item.to_dict(), item._id) for item in users], 
+            'total_feedback': total,
+            'approved': sorted(_approved, key=lambda d: d['publishedon'], reverse=True),
+            'total_approved': len(_approved),
+        }
 
 
 class ApiContentSubscribers(Resource):
