@@ -1,12 +1,14 @@
 #coding: utf-8
-from flask import current_app, g
+from flask import current_app, g, request, abort
 from flask_login import current_user
 from jinja2.ext import Extension
 from jinja2.nodes import List, CallBlock
 from esentity.models import Page
 from loguru import logger
-from json import dumps
+from json import dumps, loads
 import html
+import math
+from urllib.parse import unquote
 
 
 class HtmlExtension(Extension):
@@ -312,6 +314,7 @@ class RatingExtension(Extension):
             country=current_user.country_full,
             services=service,
             provider_tags=tags,
+            _locale=current_app.config['BABEL_DEFAULT_LOCALE'],
             _source = [
                 "title", 
                 "alias", 
@@ -500,6 +503,7 @@ class TopCasinosExtension(Extension):
             country=current_user.country_full,
             is_searchable=True,
             is_redirect=False,
+            _locale=current_app.config['BABEL_DEFAULT_LOCALE'],
             _source = [
                 "title", 
                 "alias", 
@@ -712,6 +716,7 @@ class CasinosExtension(Extension):
                 tags='casino',
                 provider_tags=tags,
                 locale_available=g.language,
+                _locale=current_app.config['BABEL_DEFAULT_LOCALE'],
                 _source=True, 
                 _count=count,
             )
@@ -771,6 +776,7 @@ class SlotsExtension(Extension):
                 tags='casino',
                 provider_tags=tags,
                 locale_available=g.language,
+                _locale=current_app.config['BABEL_DEFAULT_LOCALE'],
                 _source=True, 
                 _count=count,
             )
@@ -921,3 +927,189 @@ class CompareButtonExtension(Extension):
             parser.stream.skip_if('comma')
 
         return CallBlock(self.call_method("_render", args=[List(args)]), [], [], []).set_lineno(lineno)          
+
+
+class ComparatorExtension(Extension):
+    tags = set(['comparator'])
+
+    def __init__(self, environment):
+        super(ComparatorExtension, self).__init__(environment)
+
+    def _render(self, args, caller=None):
+        logger.info('Args: {0}'.format(args))
+
+        try:
+            casinos = []
+            if request.cookies.get('comparator'):
+                cmp_list = loads(unquote(request.cookies.get('comparator')))
+                logger.info(f'Casinos for compare: {cmp_list}')
+
+                casinos, _ = Page.get(
+                    category='provider',
+                    alias=[item['alias'] for item in cmp_list],
+                    is_active=True,
+                    is_searchable=True,
+                    is_redirect=False,  
+                    locale=current_app.config['BABEL_DEFAULT_LOCALE'], 
+                    _count=3,
+                )
+
+            t = current_app.jinja_env.get_template('_ext-comparator.html')
+            attrs = {
+                'General': [
+                    'logo',
+                    'path',
+                    'establishedon',
+                    'website',
+                    'languages',
+                    'services',
+                    'licences',
+                    'provider_pros',
+                    'provider_cons',
+                    'rating',
+                    'user_rating',
+                    'rank_alexa',
+                    'geo',
+                    'path',
+                ],
+                'Support': [
+                    'support_languages',
+                    'support_livechat',
+                    'support_email',
+                    'support_phone',
+                    'support_worktime',
+                    'logo',
+                    'path',
+                ],
+                'Banking': [
+                    'currencies',
+                    'deposits',
+                    'withdrawal',
+                    'min_deposit',
+                    'min_withdrawal',
+                    # 'withdrawal_time',
+                    'withdrawal_monthly',
+                    'kyc_deposit',
+                    'kyc_withdrawal',
+                    'path',
+                ],
+                'Games': [
+                    'games',
+                    'software',
+                    'games_count',
+                    'path',
+                ],
+                'Promotions': [
+                    'welcome_package',
+                    'welcome_package_note',
+                    'welcome_package_wager',
+                    'path',
+                ],
+            }
+
+            names = {
+                'logo': '',
+                'establishedon': 'Launched',
+                'website': 'Website',
+                'languages': 'Interface',
+                'services': 'Services',
+                'licences': 'Licences',
+                'path': '',
+                'provider_pros': 'Pros',
+                'provider_cons': 'Cons',
+                'rating': 'Editor Rating',
+                'user_rating': 'Players Rating',
+                'rank_alexa': 'Alexa Rank',
+                'geo': 'Acceptable Countries',
+                'support_languages': 'Support Languages',
+                'support_livechat': 'Livechat Available',
+                'support_email': 'Support E-mail',
+                'support_phone': 'Support Phone',
+                'support_worktime': 'Support Worktime',
+                'currencies': 'Currencies',
+                'deposits': 'Deposit Methods',
+                'withdrawal': 'Withdrawal Methods',
+                'min_deposit': 'Minimal Deposit',
+                'min_withdrawal': 'Minimal Withdrawal',
+                'withdrawal_time': 'Withdrawal Time',
+                'withdrawal_monthly': 'Withdrawal Monthly Limit',
+                'kyc_deposit': 'KYC Deposit',
+                'kyc_withdrawal': 'KYC Withdrawal',
+                'games': 'Games types',
+                'software': 'Game Providers',
+                'games_count': 'Games count',
+                'welcome_package': 'Welcome Package',
+                'welcome_package_note': 'Requirements',
+                'welcome_package_wager': 'Wager',
+            }
+
+            s = t.render(casinos=casinos, attrs=attrs, names=names, currency_geo=g.currency_geo)
+            return s
+        except Exception as e:
+            logger.error(f'Comparator Exception: {e}')
+
+        return ''
+
+    def parse(self, parser):
+        lineno = next(parser.stream).lineno
+
+        args = []
+        while parser.stream.current.type != 'block_end':
+            args.append(parser.parse_expression())
+            parser.stream.skip_if('comma')
+
+        return CallBlock(self.call_method("_render", args=[List(args)]), [], [], []).set_lineno(lineno)          
+
+
+class BlogExtension(Extension):
+    tags = set(['blog'])
+
+    def __init__(self, environment):
+        super(BlogExtension, self).__init__(environment)
+
+    def _render(self, args, caller=None):
+        logger.info('Args: {0}'.format(args))
+
+        page = 1
+        p = request.args.get('page')
+        if p:
+            if p.isdigit():
+                page = int(request.args.get('page'))
+            else:
+                abort(404)
+
+        _cpp = 12
+        pages, total = Page.get(
+            is_active=True,
+            is_searchable=True, 
+            is_redirect=False, 
+            locale=current_app.config['BABEL_DEFAULT_LOCALE'], 
+            tags='Blog',
+            _source=['path', 'title', 'publishedon', 'cover'],
+            _count=_cpp,
+            _offset=(page - 1) * _cpp,
+            _sort=[
+                {'publishedon': {'order': 'desc'}}
+            ]
+        )
+
+        logger.debug('Page: {0}'.format(page))
+        logger.debug('Total: {0}'.format(total))
+
+        if (page - 1) * _cpp > total or (request.args.get('page') and page < 2):
+            abort(404)
+
+        paging = [{'page': item, 'value': item} for item in range(1, math.ceil(total / _cpp) + 1)]
+
+        t = current_app.jinja_env.get_template('_ext-blog.html')
+        return t.render(pages=pages, paging=paging, current=page)
+
+    def parse(self, parser):
+        lineno = next(parser.stream).lineno
+
+        args = []
+        while parser.stream.current.type != 'block_end':
+            args.append(parser.parse_expression())
+            parser.stream.skip_if('comma')
+
+        return CallBlock(self.call_method("_render", args=[List(args)]), [], [], []).set_lineno(lineno)          
