@@ -1,12 +1,13 @@
 #coding:utf-8
 from flask import current_app, url_for, request
 from flask_login import AnonymousUserMixin, current_user
+from pydantic.types import constr
 from user_agents import parse
 from blinker import Signal
 from datetime import datetime, date
 from slugify import slugify 
 from loguru import logger
-from json import dumps
+from json import dumps, loads
 from urllib.parse import unquote
 import random
 import string
@@ -18,6 +19,7 @@ import hashlib
 import shortuuid
 from pydantic import BaseModel, validator, EmailStr, conint, conlist, HttpUrl
 from typing import Optional, List
+from cryptography.fernet import Fernet
 
 
 class UndefinedEntityAttribute(Exception):
@@ -155,8 +157,8 @@ class ElasticEntity(BaseEntity):
             must = [{"terms" if isinstance(v, list) else "term": {pre_field(k, v): v}} for k, v in kwargs.items()]
 
             if _range:
-                _f, _s, _e = _range
-                must.append({"range": {_f: {"gte": _s, "lte": _e}}})
+                _f, _s, _e, _tz = _range
+                must.append({"range": {_f: {"gte": _s, "lte": _e, "time_zone": _tz}}})
 
             qbool = {
                 "must": must
@@ -645,7 +647,7 @@ class Page(ElasticEntity):
             'countries': [{'iso': item.alpha_2, 'country': item.name} for item in pycountry.countries],
             'countries_name': [item.name for item in pycountry.countries],
             'services': ['casino', 'poker', 'betting', 'lotto', 'bingo'],
-            'status': ['draft', 'on_review', 'published'],
+            'status': ['published', 'draft', 'on_review'],
             'category': ['provider', 'slot', 'page', 'collection'],
             'collection_category': ['provider', 'slot'],
             'wager_types': ['xb', 'x(d+b)'],
@@ -694,7 +696,7 @@ class Page(ElasticEntity):
                 'slot_features:select:slot_features',
                 'is_jackpot:bool',
             ],
-            'collection_sort': ['Func_1', 'Func_2', 'Rate', 'Rank', 'Views', 'Activity', 'Alexa', 'Adding', 'AZ', 'Released', 'RTP', 'MaxWin', 'Jackpot'],
+            'collection_sort': ['Func_1', 'Func_2', 'Rate', 'Rank', 'Views', 'Activity', 'Alexa', 'Adding', 'AZ', 'Launched', 'RTP', 'MaxWin', 'Jackpot', 'Min.Deposit'],
             'collection_sort_direction': ['Desc', 'Asc'],
             'collection_paging': ['FirstPage', 'Paging'],
         }
@@ -962,6 +964,7 @@ class Actor(ElasticEntity, Guest):
         'username': str,
         'password': str,
         'last_auth': DateTimeField,
+        'last_country': str,
         'actor_is_active': bool,
         'actor_is_admin': bool,
         'zones': list,
@@ -1121,6 +1124,27 @@ class TdsHit(ElasticEntity):
                     })
                 },
             }        
+        elif _field in ['stream_key']:
+            aggs = {
+                _field: {
+                    "terms": {
+                        "field": 'stream.keyword',
+                        "size": _cnt
+                    },
+                    "aggs": get_subaggs({
+                        "campaign_name": {
+                            "terms": {
+                                "field": "campaign_name.keyword"
+                            },
+                        },
+                        "stream_name": {
+                            "terms": {
+                                "field": "stream_name.keyword"
+                            },
+                        },
+                    })
+                },
+            }        
         else:
             aggs = {
                 _field: {
@@ -1135,8 +1159,8 @@ class TdsHit(ElasticEntity):
         must = []
 
         if _range:
-            _f, _s, _e = _range
-            must.append({"range": {_f: {"gte": _s, "lte": _e}}})
+            _f, _s, _e, _tz = _range
+            must.append({"range": {_f: {"gte": _s, "lte": _e, "time_zone": _tz}}})
 
         q = {
             "query": {
@@ -1172,8 +1196,10 @@ class TdsHit(ElasticEntity):
                     return [{'term': item['key_as_string'].strip(), 'hits': item['doc_count'], 'uc': get_uc(item), 'bots': get_bots(item)} for item in items[key]['buckets']]
                 elif key == 'country':
                     return [{'term': item['key'].strip(), 'country_iso': item['country_iso']['buckets'][0]['key'].lower(), 'hits': item['doc_count'], 'uc': get_uc(item), 'bots': get_bots(item)} for item in items[key]['buckets']]
-                elif key == 'campaign_id':
+                elif key in ['campaign_id']:
                     return [{'term': ', '.join([k['key'] for k in item['campaign_name']['buckets']]), 'hits': item['doc_count'], 'uc': get_uc(item), 'bots': get_bots(item)} for item in items[key]['buckets']]
+                elif key in ['stream_key']:
+                    return [{'term': item['key'].strip() + ' | Name: ' + ', '.join([k['key'] for k in item['stream_name']['buckets']]) + ' | Campaign: ' + ', '.join([k['key'] for k in item['campaign_name']['buckets']]), 'hits': item['doc_count'], 'uc': get_uc(item), 'bots': get_bots(item)} for item in items[key]['buckets']]
                 else:
                     return [{'term': item['key'].strip(), 'hits': item['doc_count'], 'uc': get_uc(item), 'bots': get_bots(item)} for item in items[key]['buckets']]
             return []
@@ -1222,7 +1248,7 @@ class TdsCampaign(ElasticEntity):
 # pydantic models
 
 class UserModel(BaseModel):
-    login: str = ...
+    login: EmailStr
 
     @validator('login', allow_reuse=True)
     def fn_not_empty(cls, v):
@@ -1231,7 +1257,7 @@ class UserModel(BaseModel):
 
 
 class ManagerModel(BaseModel):
-    login: str = ...
+    login: EmailStr
     skype: str = ...
 
     @validator('login', 'skype', allow_reuse=True)
@@ -1241,33 +1267,45 @@ class ManagerModel(BaseModel):
 
 
 class SignupModel(BaseModel):
-    login: str = ...
+    login: EmailStr
     skype: str = ...
-    invite_code: str = ...
+    invite_code: Optional[str] = None
 
     @validator('login', 'skype', allow_reuse=True)
     def fn_not_empty(cls, v):
         assert v != '', 'Required field'
         return v
 
+
+class SignupModelInvite(SignupModel):
+    invite_code: str = ...
+
     @validator('invite_code', allow_reuse=True)
     def fn_invite_code(cls, v):
-        def is_valid(s):
-            if len(s) == 56:
-                objs, total = Page.get(_id=s)
-                if total == 1:
-                    obj = objs.pop()
-                    if obj.category == 'provider':
-                        return True
-            return False
-        
         assert v != '', 'Required field'
-        assert v == current_app.config['DASHBOARD_INVITE'] or is_valid(v), 'Invite Expired'
-        return v
+        if v == current_app.config['DASHBOARD_INVITE']:
+            return v
+
+        try:
+            f = Fernet(current_app.config['FERNET_KEY'])
+            _obj = f.decrypt(v.encode())
+            _token = loads(_obj)
+            if isinstance(_token, dict):
+                _ts = datetime.timestamp(datetime.utcnow())
+                if _ts > _token['expired']:
+                    logger.info(f'Token expired: {_token}, now: {_ts}')
+                    assert False, 'Invite Expired'
+                else:
+                    logger.info(f'Token valid: {_token}')
+                    return v
+        except Exception as e:
+            logger.error(f'Exception encrypt token: {e}')
+
+        assert False, 'Invite Expired'
 
 
 class AuthModel(BaseModel):
-    login: str = ...
+    login: EmailStr
     password: str = ...
 
     @validator('login', 'password', allow_reuse=True)
@@ -1319,8 +1357,8 @@ class CasinoModel(BaseModel):
     welcome_package_max_bonus: Optional[int] = None
     welcome_package_match: Optional[int] = None
     welcome_package_fs: Optional[int] = None
-    welcome_package: str
-    welcome_package_note: Optional[str] = None
+    welcome_package: constr(max_length=50)
+    welcome_package_note: Optional[constr(max_length=500)] = None
     welcome_package_wager: Optional[int] = None
     cashback: Optional[int] = None
     geo_whitelist: Optional[str] = None
@@ -1329,6 +1367,15 @@ class CasinoModel(BaseModel):
     content: Optional[str] = None
 
 
+class CasinoProxyModel(CasinoModel):
+    website: Optional[str] = None
+    default_currency: Optional[str] = None
+    min_deposit: Optional[int] = None
+    min_withdrawal: Optional[int] = None
+    welcome_package: Optional[str] = None
+    welcome_package_note: Optional[str] = None
+
+
 class DomainModel(BaseModel):
     @validator('*', pre=True, allow_reuse=True)
     def empty_str_to_none(cls, v):
@@ -1388,4 +1435,56 @@ class ActivityFeedbackModel(BaseModel):
     casino_id: str = ...
     name: Optional[str]
     pros: Optional[str]
-    cons: Optional[str]                
\ No newline at end of file
+    cons: Optional[str]                
+
+
+class CasinoCommit(ElasticEntity):
+    on_put = Signal()
+
+    _schemas = {
+        'updatedon': DateTimeField,
+        'source_id': str,
+        'actor_id': str,
+        'casino': str,
+        ###
+        'title': str,
+        'website': str,
+        'languages': list,
+        'services': list,
+        'year': int,
+        'support_languages': list,
+        'support_livechat': bool,
+        'support_email': str,
+        'support_phone': str,
+        'attachs': list,
+        'logo': str,
+        'logo_small': str,
+        'licences': list,
+        'games': list,
+        'games_count': int,
+        'software': list,
+        'currencies': list,
+        'default_currency': str,
+        'deposits': list,
+        'min_deposit': int,
+        'kyc_deposit': bool,
+        'withdrawal': list,
+        'min_withdrawal': int,
+        'withdrawal_monthly': int,
+        'kyc_withdrawal': bool,
+        'welcome_package_max_bonus': int,
+        'welcome_package_match': int,
+        'welcome_package_fs': int,
+        'welcome_package': str,
+        'welcome_package_note': str,
+        'welcome_package_wager': int,
+        'cashback': int,
+        'geo_whitelist': str,
+        'geo_blacklist': str,
+        'geo': list,
+        'content': str,
+    }
+        
+    @classmethod
+    def _table(cls):
+        return "{0}_casino_commit.{1}".format(os.environ.get('PROJECT', 'project'), os.environ.get('VERSION', 'v1'))
